#!/usr/bin/env python
# -*- coding: utf-8 -*-

# GIMP plugin to create a painting on satin

# (c) Dinasset 2022
#
#   History:
#
#   v1.0: 2022-01: Initial version
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your engraveL) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import sys, os
import time
import datetime
from gimpfu import *

# define here some constant value, when required
#FixWH=2000   # now dinamic

############################################################################
# Service routines
#---------------------------------------------------------------------------
  
#---------------------------------------- 
# for messages use: erro_msg or info_msg
#---------------------------------------- 
def erro_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(0) # box
    pdb.gimp_message ("ERROR: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###
def info_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2) # error console
    pdb.gimp_message ("INFO: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###

#----------------------------------------------- 
# from Original drawable create the Output image
#----------------------------------------------- 
def createWrkimage(inImage, inLayer, WrkWH):
     
    owidth = inImage.width
    oheight = inImage.height
    otype = inImage.base_type
    oname = inLayer.name
    OrigWH = inLayer.width+inLayer.height    
    
   # define/calculate the scale factor for the (new) output image  
    scale = (WrkWH*1.0)/(OrigWH*1.0)      # ratio for scaling

    rwidth=int(owidth*scale)
    rheight=int(oheight*scale)
    
   # create a new image of the decided size  
   # --- copy the source layer
   # --- scale
    newImage = pdb.gimp_image_new (rwidth, rheight, otype)                    
    newBG = pdb.gimp_layer_new_from_drawable(inLayer, newImage)     # background layer
    newBG.mode = LAYER_MODE_NORMAL
    newBG.name = "NewImage: Background Layer from "+oname
    newBG.opacity = 100.0 
    newImage.add_layer(newBG, 0)
    pdb.gimp_layer_scale(newBG, rwidth, rheight, False)
    pdb.gimp_layer_add_alpha (newBG)    # RGBA
    
    # adjust mean values
    adjustmeanv (newImage, newBG)
    
   # keep a copy of the original background below the one which will be used ----------------
   # --- and set its visibility off (it should not enter in the process)
    Original = newBG.copy()
    newImage.add_layer(Original, 1)     # below
    pdb.gimp_item_set_visible (Original, False)
    Original.name = oname+" (original)"
   
    newBG.opacity = 100.0
    
    return(newImage)
#---------------------------------------------------------------------------

#---------------------------------------------------------------------------
# adjust values and saturation
#---------------------------------------------------------------------------
def adjustmeanSV (inImage, inLayer):
    # decompose HSV
    (HSVI,HL,SL,VL) = pdb.plug_in_decompose (inImage, inLayer, "HSV", True)
    #pdb.gimp_display_new (HSVI)
    HL=HSVI.layers[0]; SL=HSVI.layers[1]; VL=HSVI.layers[2];
    adjustmeanv (HSVI, SL)
    adjustmeanv (HSVI, VL)
    pdb.plug_in_recompose (HSVI, HL)
    return(inLayer)

#---------------------------------------------------------------------------
# adjust values
#---------------------------------------------------------------------------
def adjustmeanv(inImage, inLayer):
    meanadj=50.0
    # take care of the existing mean in determining the theoretical mean 
    # check mean value of the layer
    (meanl,std,med,pix,cnt,per) = pdb.gimp_drawable_histogram (inLayer,
                                  HISTOGRAM_VALUE, 0.0, 1.0)
    thmean = (127.5*meanadj/100+meanl*(100.0-meanadj)/100)   
    #info_msg ("thmean="+str(thmean))
    # range for the loops
    minmean = thmean*1.05  
    maxmean = thmean*1.15  
    #info_msg ("target min mean value="+str(minmean))
    #info_msg ("target max mean value="+str(maxmean))
    
    if meanl<minmean:
        while meanl<minmean:
            #info_msg ("mean source value="+str(meanl))
            pdb.gimp_drawable_levels (inLayer, HISTOGRAM_VALUE,
                                      0.0, 1.0, False, 1.1,
                                      0.0, 1.0, False)
            (meanl,std,med,pix,cnt,per) = pdb.gimp_drawable_histogram (inLayer,
                                          HISTOGRAM_VALUE, 0.0, 1.0)
    elif meanl>maxmean:
        while meanl>maxmean:
            #info_msg ("mean source value="+str(meanl))
            pdb.gimp_drawable_levels (inLayer, HISTOGRAM_VALUE,
                                      0.0, 1.0, False, 0.9,
                                      0.0, 1.0, False)
            (meanl,std,med,pix,cnt,per) = pdb.gimp_drawable_histogram (inLayer,
                                          HISTOGRAM_VALUE, 0.0, 1.0)
    
    
    return(inLayer)


############################################################################
# Main
#---------------------------------------------------------------------------

def EdgesEngraveB (inImage, inLayer,
                  UserOutWH,
                  PrepareInput, EngraveLook, Preset,
                  doFlatten):
    
    
    # here the logic of the filter
    oname = inLayer.name
    if "." in oname:
        split = oname.split(".", 1)
        oname = split[0]
    OrigWH = inLayer.width+inLayer.height    
    
    ts = time.time()
    st = datetime.datetime.fromtimestamp(ts).strftime("%Y-%m-%d %H-%M-%S")
    
    # create new image to work on
    WrkWH=2000+1000*int(EngraveLook)
    newImage = createWrkimage (inImage, inLayer, WrkWH)
    width = newImage.width
    height = newImage.height
    Preprocessed = newImage.layers[0]
    Preprocessed.name = "Preprocessed"
    # Apply a Paint look
    # thru Segmentation  
    pdb.plug_in_gmic_qt(newImage, Preprocessed, 1,2,
    '-v - -fx_segment_watershed 9,2,0,0,0,50,50 ')
    Painted = newImage.layers[0]
    Painted.mode = LAYER_MODE_NORMAL
    Painted.opacity = 100.0
    Painted.name = "Preprocessed painted thru Segmentation"
    pdb.plug_in_unsharp_mask (newImage, Painted, 3.0, 3.0, 0)
    
    #pdb.gimp_display_new (newImage)
    #return
    
    # check whether a preparation required
    if PrepareInput==1: # Apply Fond Brosse: samj_Fond_Brosse 3,1
        FBH = Painted.copy()
        newImage.add_layer (FBH, 0)
        pdb.plug_in_gmic_qt(newImage, FBH, 1,0,
            '-v - -samj_Fond_Brosse 3,1 ')
        FBV = Painted.copy()
        newImage.add_layer (FBV, 0)
        pdb.gimp_item_transform_rotate_simple (FBV, ROTATE_90, True,
                                                   int(width/2), int(height/2))
        pdb.plug_in_gmic_qt(newImage, FBV, 1,0,
            '-v - -samj_Fond_Brosse 3,1 ')
        pdb.gimp_item_transform_rotate_simple (FBV, ROTATE_270, True,
                                                   int(width/2), int(height/2))
        FBV.opacity = 50.0
        FBHV = pdb.gimp_image_merge_down (newImage, FBV, CLIP_TO_IMAGE)
        FBHV.name = "Applied Fond Brosse H+V"
        
    elif PrepareInput==2: # Apply 
                          # samj_Contours_Blancs 1,16,2,0,0.5,0,100,0,0,3,1,0,0,0,15,4,1,0
        CB = Painted.copy()
        newImage.add_layer (CB, 0)
        pdb.gimp_layer_scale(CB, width/3, height/3, False)
        pdb.plug_in_gmic_qt(newImage, CB, 1,2,
            '-v - -samj_Contours_Blancs 1,16,2,0,0.5,0,100,0,0,3,1,0,0,0,15,4,1,0 ')
        CBL = newImage.layers[0]
        pdb.gimp_layer_scale(CB, width, height, False)
        pdb.gimp_layer_scale(CBL, width, height, False)
        FBHV = pdb.gimp_image_merge_down (newImage, CBL, CLIP_TO_IMAGE)
        FBHV.name = "Applied Contours_Blancs"
        
    else:
        FBHV = Painted.copy()
        newImage.add_layer (FBHV, 0)
        FBHV.name = "no preparation"
        
    FBHV.opacity = 50.0
    FBHV.mode = LAYER_MODE_OVERLAY    
    
    #pdb.gimp_display_new (newImage)
    #return
    
    # Apply samj_NB_EdgesO_Engrave 0,50,9,1,1,50,0,8,40,0,0,1
    #       samj_NB_EdgesO_Engrave 0,50,3,1,2,50,0,40,40,0,10,1
    EdgesEngraveparms = '0,50,3,1,2,50,0,40,40,0,10,1'
    pdb.plug_in_gmic_qt(newImage, FBHV, 1,2,
    '-v - -samj_NB_EdgesO_Engrave '+EdgesEngraveparms+' ')
    EdgesEngrave = newImage.layers[0]
    EdgesEngrave.mode = LAYER_MODE_NORMAL   # temp HSV_VALUE   
    EdgesEngrave.opacity = 100.0    # temp 
    EdgesEngrave.name = "samj_NB_EdgesO_Engrave "+EdgesEngraveparms
    
    #pdb.gimp_display_new (newImage)
    #return
    
    # fill blacks with pattern
    pdb.gimp_image_select_color (newImage, CHANNEL_OP_REPLACE,
                                 EdgesEngrave, (0,0,0,255))
    pdb.gimp_context_set_pattern ("Paper")
    pdb.gimp_drawable_edit_fill (EdgesEngrave, FILL_PATTERN)
    # fill interior with white
    pdb.gimp_selection_shrink (newImage, 1)
    pdb.gimp_drawable_edit_fill (EdgesEngrave, FILL_WHITE)
    pdb.gimp_selection_invert (newImage)
    pdb.gimp_selection_none (newImage)
    # spread
    pdb.plug_in_spread (newImage, EdgesEngrave, width*1.0/500, height*1.0/500)
    
    # Add 2 posterized layers:
    # - first one will be desaturated, second one remains in colours
    pdb.gimp_image_set_active_layer (newImage, Painted)
    # Apply samj_Posterize_B 5,32,2.501,4,400,50,5,0,1,0,0,1,0,0,0,0,0,0
    Posterparms = '5,32,2.501,4,400,50,5,0,1,0,0,1,0,0,0,0,0,0'
    pdb.plug_in_gmic_qt(newImage, Preprocessed, 1,2,
    '-v - -samj_Posterize_B '+Posterparms+' ')
    Poster = newImage.layers[0]
    pdb.plug_in_unsharp_mask (newImage, Poster, 2.0, 2.0, 0)
    # Dup the Poster layer (add it layer)
    Poster2 = Poster.copy() # not attached here to the image, but later on
    # after the dup: set mode of first copy to act over EdgesEngrave
    Poster.mode = LAYER_MODE_LINEAR_BURN    # GRAIN_MERGE   
    Poster.opacity = 90.0   #100.0 # confirm   
    pdb.gimp_drawable_desaturate (Poster, DESATURATE_LUMA)
    Poster.name = "Posterized desaturated (for values of texture)"
    # remove blacks from desaturated poster
    pdb.gimp_drawable_levels (Poster, HISTOGRAM_VALUE,
                                      0.0, 1.0, False, 1.0,
                                      0.05, 1.0, False)
    Texture = pdb.gimp_layer_new_from_visible (newImage, newImage,
                                               "Engrave texture")
    newImage.add_layer (Texture, 0) 
    pdb.gimp_item_set_visible (EdgesEngrave, False)
    pdb.gimp_item_set_visible (Poster, False)
    Texture.mode = LAYER_MODE_GRAIN_EXTRACT #HSV_VALUE
    if Preset==0:   # standard
        Texture.opacity = 90.0   
    elif Preset==1:   # less relevance to the texture
        Texture.opacity = 45.0
    Texture.name = "Texture of Edges Engrave"
    # def for Poster with colours to act over texture
    newImage.add_layer (Poster2, 0)
    # remove blacks from colored poster
    pdb.gimp_drawable_levels (Poster2, HISTOGRAM_VALUE,
                                      0.0, 1.0, False, 1.0,
                                      0.05, 1.0, False)
    Poster2.mode = LAYER_MODE_LINEAR_BURN #OVERLAY #MULTIPLY #LINEAR_LIGHT #PIN_LIGHT #GRAIN_MERGE 
    Poster2.opacity = 65.0 #75.0  #100.0  
    Poster2.name = "Posterized in colours"
    
    # final step
    if doFlatten==True:
        outcome = pdb.gimp_image_merge_visible_layers (newImage, CLIP_TO_BOTTOM_LAYER)
        outcome.name = "Outcome"
        pdb.gimp_image_remove_layer (newImage, EdgesEngrave)
        pdb.gimp_image_remove_layer (newImage, Poster)
    else:    
        outcome = pdb.gimp_layer_new_from_visible (newImage, newImage, "Outcome")
        newImage.add_layer (outcome, 0)
        pdb.gimp_item_set_visible (outcome, False)
        
    outcome.name = st+oname+" with an edges engrave look (B) "
    prepareds = ["none", "Fond Brosse", "Contours Blancs" ]
    preparedused = " (preparation "+prepareds[PrepareInput]+")"
    engraveLs = ["basic", "regular", "fine" ]
    engraveLused = " (texture look "+engraveLs[EngraveLook]+")"
    presets = ["standard", "less texture evidence" ]
    presetused = " (preset "+presets[Preset]+")"
    outcome.name = outcome.name+preparedused+engraveLused+presetused
    
    # user required output image size
    if UserOutWH==0: UserOutWH=OrigWH
    if UserOutWH != WrkWH:
        uRatio = UserOutWH*1.00/WrkWH*1.00
        uWidth = int(width*uRatio)
        uHeight = int(height*uRatio)
        #info_msg ("SCALING")   # not during tests, scaling is slow
        pdb.gimp_image_scale (newImage, uWidth, uHeight)   # to user chosen size
    
    pdb.gimp_display_new (newImage)

### Registration
whoiam="\n"+os.path.abspath(sys.argv[0])
author="dinasset"
menu="<Image>/Diego/SimpleGraphics/"
copyrightYear="2022"
desc="Edges Engrave B "

register(
    "EdgesEngraveB",
    desc+whoiam,desc,
    author,author,
    copyrightYear,
    desc+" ...",
    "*",
    [
        (PF_IMAGE, "image", "Input image", None),
        (PF_DRAWABLE, "drawable", "Input drawable", None),
        (PF_ADJUSTMENT, "newsize", "Width+Height of New Image"+
                                   "\n (0=same as input)",
                                   3000, (0, 7200, 100)),
        (PF_OPTION, "prepare", "Prepared input", 1, ["*none", "Fond Brosse", "Contours Blancs" ]),
        (PF_OPTION, "lines", "Lines look", 1, ["Basic", "Regular", "Fine" ]),
        (PF_OPTION, "preset", "Layers preset", 1, ["Standard", "Less texture relevance" ]),
        (PF_TOGGLE, "flatten", "Flatten Image?", False),
    ],
    [],
    EdgesEngraveB,
    menu=menu
)

main()
